unit ICQSock;
(***********************************************************)
(*   TICQSock Object, v. 01 (initial release)
(*   TICQSock uses async. windows sockets, avaible fnct:
(*   Send; OnError, OnReceive and OnTimeout events.
(*
(*   Written by Coban(alex@ritlabs.com), special for using
(*   in ICQ Password Whacker app.
(**********)
interface
uses
  Messages,
  Windows,
  WinSock,
  Dialogs,
  SysUtils,
  AllocateRes;

const
  WM_SOCKET = WM_USER + $250;
  WM_TIMER_EVENT = WM_USER + $212;
  WM_ACK_EVENT = WM_USER + $412;
  WSA_VER_REQUIRED = $0101;
  MAX_BUF_SIZE = 450;

type
  TUDPOnErrorProc = procedure(ErrorStr: String; ErrorNo: Integer) of object;
  TUDPOnReceiveProc = procedure(var Buf: array of Byte; BufLen: Integer) of object;
  TUDPOnTimeoutProc = procedure of object;
  TWndProc = procedure(var Message: TMessage) of object;

  TSockMsg = record
    Msg: Cardinal;
    SockID: THandle;
    SelectEvent: Word;
    SelectError: Word;
    Result: LongInt;
  end;

  TICQSock = class
  private
    LPort: Integer;
    RPort: Integer;
    RHost: String;
    FWSAData: TWSAData;
    FWinHandle: THandle;
    FOnError: TUDPOnErrorProc;
    FOnReceive: TUDPOnReceiveProc;
    FOnTimeout: TUDPOnTimeoutProc;
    FOnAckTimeout: TUDPOnTimeoutProc;
    FSrvSocket: TSocket;
    FTimer: THandle;
    FAckTimer: THandle;
    FBinded: Boolean;
  protected
    wClass: TWndClass;
    procedure SockProc(var Msg: TMessage);
    procedure SockEvent(var Msg: TSockMsg); message WM_SOCKET;
    procedure StartWSA;
    procedure CloseWSA;
    function ResolveAddr(var _InAddr: String): String;
    function GetData(var Buf; buf_len: Integer; var bytes_read: Integer): Integer;
    function CreateSrvSocket(var sock: TSocket; port: Integer): TSocket;
    function UnBindSrvSocket(var sock: TSocket): TSocket;
    procedure _TimerProc(var Msg: TMsg); message WM_TIMER;
    function _SetTimer(timeout: Integer): THandle;
    function _SetAckTimer(timeout: Integer): THandle;
    procedure _RaiseWSAException;
  public
    constructor Create(LocalPort: Integer; RemotePort: Integer; RemoteHost: String);
    destructor Destroy; override;
    procedure ReBind(LocalPort: Integer);
    procedure Send(var Buf; Len: Integer; timeout: Integer = 0; ack_timeout: Integer = 0);
  published
    property IsBinded      : Boolean Read FBinded;
    property OnError       : TUDPOnErrorProc Read FOnError Write FOnError;
    property OnReceive     : TUDPOnReceiveProc Read FOnReceive Write FOnReceive;
    property OnTimeout     : TUDPOnTimeoutProc Read FOnTimeout Write FOnTimeout;
    property OnAckTimeout  : TUDPOnTimeoutProc Read FOnAckTimeout Write FOnAckTimeout;
  end;

implementation
  constructor TICQSock.Create(LocalPort: Integer; RemotePort: Integer; RemoteHost: String);
  begin
    LPort := LocalPort;
    RPort := RemotePort;
    RHost := RemoteHost;
    FWinHandle := 0;
    while FWinHandle = 0 do
      FWinHandle := AllocateHWND(SockProc);
    StartWSA;
    if inet_addr(PChar(RHost)) = LongInt(INADDR_NONE) then
      RHost := ResolveAddr(RHost);
    CreateSrvSocket(FSrvSocket, LPort);
  end;

  destructor TICQSock.Destroy;
  begin
    UnBindSrvSocket(FSrvSocket);
    DeallocateHWND(FWinHandle);
    CloseWSA;
    inherited Destroy;
  end;

  procedure TICQSock.ReBind(LocalPort: Integer);
  begin
    LPort := LocalPort;
    UnBindSrvSocket(FSrvSocket);
    KillTimer(FWinHandle, WM_TIMER_EVENT);
    KillTimer(FWinHandle, WM_ACK_EVENT);
    CreateSrvSocket(FSrvSocket, LPort);
//      _RaiseWSAException;
  end;

  procedure TICQSock.SockProc(var Msg: TMessage);
  begin
    case Msg.Msg of
      WM_SOCKET, WM_TIMER:
          Dispatch(Msg);
      end;
  end;

  procedure TICQSock.SockEvent(var Msg: TSockMsg);
  var
    buf: array[0..MAX_BUF_SIZE] of Byte;
    bytes_read: Integer;
  begin
    if Msg.SelectError <> NOERROR then
      _RaiseWSAException
    else
    begin
      case Msg.SelectEvent of
         FD_READ   :
         begin
           FillChar(Buf, SizeOf(Buf), 0);
           if GetData(buf, SizeOf(buf), bytes_read) > 0 then
             begin
               if Assigned(FOnReceive) then
                 FOnReceive(buf, bytes_read)
             end
         end;
      end;
    end;
  end;

  procedure TICQSock.StartWSA;
  begin
    if WSAStartup(WSA_VER_REQUIRED, FWSAData) <> NOERROR then
      _RaiseWSAException;
  end;

  procedure TICQSock.CloseWSA;
  begin
    if WSACleanup <> NOERROR then
      _RaiseWSAException;
  end;

  function TICQSock.ResolveAddr(var _InAddr: String): String;
  var
    addr: in_addr;
    hostent: PHostEnt;
    inaddr_buf: array[0..255] of Char;
  begin
    Result := '';
    StrPCopy(inaddr_buf, _InAddr);
    hostent := GetHostByName(inaddr_buf);
    if hostent <> nil then
    begin
      addr.S_addr := LongInt(PLongInt(hostent^.h_addr_list^)^);
      Result := inet_ntoa(addr);
    end;
  end;

  function TICQSock.GetData(var Buf; buf_len: Integer; var bytes_read: Integer): Integer;
  var
    FromAddr: TSockAddrIn;
    FromAddrSize: Integer;
  begin
    FillChar(FromAddr, SizeOf(FromAddr), 0);
    FromAddr.sin_family := AF_INET;
    FromAddr.sin_port := htons(LPort);
    FromAddrSize := SizeOf(FromAddr);
    FillChar(buf, SizeOf(buf), 0);
    bytes_read := recvfrom(FSrvSocket, buf, buf_len, 0, FromAddr, FromAddrSize);
    Result := bytes_read;
    if bytes_read = SOCKET_ERROR then
      _RaiseWSAException;
  end;

  function TICQSock.CreateSrvSocket(var sock: TSocket; port: Integer): TSocket;
  var
    srv_addr: sockaddr_in;
    protoent: PProtoEnt;
    s: Integer;
  begin
    Result := NOERROR;
    FillChar(protoent, SizeOf(protoent), 0);
    protoent := GetProtoByName('udp');
    sock := socket(PF_INET, SOCK_DGRAM, protoent^.p_proto);
    FillChar(srv_addr, SizeOf(srv_addr), 0);
    srv_addr.sin_family := AF_INET;
    srv_addr.sin_port := htons(port);
    srv_addr.sin_addr.S_addr := 0;
    s := bind(sock, srv_addr, sizeof(srv_addr));
    if s <> SOCKET_ERROR then
    begin
      if WSAAsyncSelect(sock, FWinHandle, WM_SOCKET, FD_READ) = 0 then
          FBinded := True
        else
        begin
          FBinded := False;
          _RaiseWSAException;
        end;
    end
    else
      FBinded := False;
    if s = SOCKET_ERROR then
    begin
      Result := SOCKET_ERROR;
      _RaiseWSAException;
    end;
  end;

  function TICQSock.UnBindSrvSocket(var sock: TSocket): TSocket;
  begin
    Result := NOERROR;
    if FBinded then
      if CloseSocket(FSrvSocket) <> NOERROR then
      begin
        Result := SOCKET_ERROR;
        _RaiseWSAException;
      end
      else
        FBinded := False;
  end;

  procedure TICQSock.Send(var Buf; Len: Integer; timeout: Integer = 0; ack_timeout: Integer = 0);
  var
    snd_addrin: sockaddr_in;
  begin
    if FBinded then
    begin
      FillChar(snd_addrin, SizeOf(snd_addrin), 0);
      snd_addrin.sin_family := AF_INET;
      snd_addrin.sin_port := htons(RPort);
      snd_addrin.sin_addr.S_addr := inet_addr(PChar(RHost));
      if sendto(FSrvSocket, Buf, Len, 0, snd_addrin, SizeOf(snd_addrin)) = SOCKET_ERROR then
        _RaiseWSAException
      else
      begin
        if timeout <> 0 then
          _SetTimer(timeout);
        if ack_timeout <> 0 then
          _SetAckTimer(ack_timeout);
      end;
    end;
  end;

  procedure TICQSock._TimerProc(var Msg: TMsg);
  begin
    case Msg.Message of
      WM_TIMER_EVENT:
      begin
        KillTimer(FWinHandle, WM_TIMER_EVENT);
        if Assigned(FOnTimeout) then
         FOnTimeout;
      end;
      WM_ACK_EVENT:
      begin
        KillTimer(FWinHandle, WM_ACK_EVENT);
        if Assigned(FOnAckTimeout) then
          FOnAckTimeout;
      end;
    end;
  end;

  function TICQSock._SetTimer(timeout: Integer): THandle;
  begin
    FTimer := SetTimer(FWinHandle, WM_TIMER_EVENT, timeout, nil);
    Result := FTimer;
  end;

  function TICQSock._SetAckTimer(timeout: Integer): THandle;
  begin
    FAckTimer := SetTimer(FWinHandle, WM_ACK_EVENT, timeout, nil);
    Result := FAckTimer;
  end;

  procedure TICQSock._RaiseWSAException;
  var
    ErrorNo: Integer;
    ErrorStr: String[50];
  begin
    ErrorNo := WSAGetLastError;
    case ErrorNo of
      WSAEINTR:
        ErrorStr := 'Interrupted system call';
      WSAEBADF:
        ErrorStr := 'Bad file number';
      WSAEACCES:
        ErrorStr := 'Permission denied';
      WSAEFAULT:
        ErrorStr := 'Bad address';
      WSAEINVAL:
        ErrorStr := 'Invalid argument';
      WSAEMFILE:
        ErrorStr := 'Too many open files';
      WSAEWOULDBLOCK:
        ErrorStr := 'Operation would block';
      WSAEINPROGRESS:
        ErrorStr := 'Operation now in progress';
      WSAEALREADY:
        ErrorStr := 'Operation already in progress';
      WSAENOTSOCK:
        ErrorStr := 'Socket operation on non-socket';
      WSAEDESTADDRREQ:
        ErrorStr := 'Destination address required';
      WSAEMSGSIZE:
        ErrorStr := 'Message too long';
      WSAEPROTOTYPE:
        ErrorStr := 'Protocol wrong type for socket';
      WSAENOPROTOOPT:
        ErrorStr := 'Protocol not available';
      WSAEPROTONOSUPPORT:
        ErrorStr := 'Protocol not supported';
      WSAESOCKTNOSUPPORT:
        ErrorStr := 'Socket type not supported';
      WSAEOPNOTSUPP:
        ErrorStr := 'Operation not supported on socket';
      WSAEPFNOSUPPORT:
        ErrorStr := 'Protocol family not supported';
      WSAEAFNOSUPPORT:
        ErrorStr := 'Address family not supported by protocol family';
      WSAEADDRINUSE:
        ErrorStr := 'Address already in use';
      WSAEADDRNOTAVAIL:
        ErrorStr := 'Can''t assign requested address';
      WSAENETDOWN:
        ErrorStr := 'Network is down';
      WSAENETUNREACH:
        ErrorStr := 'Network is unreachable';
      WSAENETRESET:
        ErrorStr := 'Network dropped connection on reset';
      WSAECONNABORTED:
        ErrorStr := 'Software caused connection abort';
      WSAECONNRESET:
        ErrorStr := 'Connection reset by peer';
      WSAENOBUFS:
        ErrorStr := 'No buffer space available';
      WSAEISCONN:
        ErrorStr := 'Socket is already connected';
      WSAENOTCONN:
        ErrorStr := 'Socket is not connected';
      WSAESHUTDOWN:
        ErrorStr := 'Can''t send after socket shutdown';
      WSAETOOMANYREFS:
        ErrorStr := 'Too many references: can''t splice';
      WSAETIMEDOUT:
        ErrorStr := 'Connection timed out';
      WSAECONNREFUSED:
        ErrorStr := 'Connection refused';
      WSAELOOP:
        ErrorStr := 'Too many levels of symbolic links';
      WSAENAMETOOLONG:
        ErrorStr := 'File name too long';
      WSAEHOSTDOWN:
        ErrorStr := 'Host is down';
      WSAEHOSTUNREACH:
        ErrorStr := 'No route to host';
      WSAENOTEMPTY:
        ErrorStr := 'Directory not empty';
      WSAEPROCLIM:
        ErrorStr := 'Too many processes';
      WSAEUSERS:
        ErrorStr := 'Too many users';
      WSAEDQUOT:
        ErrorStr := 'Disc quota exceeded';
      WSAESTALE:
        ErrorStr := 'Stale NFS file handle';
      WSAEREMOTE:
        ErrorStr := 'Too many levels of remote in path';
      WSASYSNOTREADY:
        ErrorStr := 'Network sub-system is unusable';
      WSAVERNOTSUPPORTED:
        ErrorStr := 'WinSock DLL cannot support this application';
      WSANOTINITIALISED:
        ErrorStr := 'WinSock not initialized';
      WSAHOST_NOT_FOUND:
        ErrorStr := 'Host not found';
      WSATRY_AGAIN:
        ErrorStr := 'Non-authoritative host not found';
      WSANO_RECOVERY:
        ErrorStr := 'Non-recoverable error';
      WSANO_DATA:
        ErrorStr := 'No Data';
      else
        ErrorStr := 'Not a WinSock error';
    end;
    if Assigned(OnError) then
      FOnError(ErrorStr, ErrorNo);
  end;
end.